## p.test is the percentage of test data to hold out from mydat
spatial.boot <- function(mydat, p.test){
  l <- nrow(mydat)
  mydat <- mydat[sample(1:l, l, replace = FALSE), ]
  mydat$index <- seq(1, l, 1)
  ## Construct unstructured variable zipcode.un
  mydat$zipcode.un <- mydat$zipcode
  
  mydat <- merge(mydat, ohio_zip_county, by.x = "zipcode", by.y = "zipcode", all.x = TRUE)
  
  ## match simulated zipcodes and those in the shapefile
  mydat$zip.un <- match(mydat$zipcode.un, X$ZCTA5CE10)
  mydat$zip <- match(mydat$zipcode, X$ZCTA5CE10)
  
  mydat$county.str <- match(mydat$County, X.county$NAME10)
  mydat$county.un <- mydat$county.str
  
  mydat <- mydat[order(mydat$index), ]
  
  mydat.pred <- mydat[((1-p.test)*l + 1):l, ]
  ## Construct indicator variable for logistic regression
  
  mydat.n <- rbind(mydat[which(mydat$index <= ((1-p.test)*l + 1) & mydat$cost > 0), ], mydat.pred)
  mydat$zero.ind <- as.numeric(mydat$cost > 0)
  ## Take log of non-zero cost for normal regression
  mydat.n$cost.log <- log(mydat.n$cost)
  
  
  
  bs.age.n <- as.data.frame(scale(bs(mydat.n$age, df = 5, degree = 3), center = TRUE, scale = FALSE))
  colnames(bs.age.n) <- c("A1", "A2", "A3", "A4", "A5")
  mydat.n <- cbind(mydat.n, bs.age.n)
  bs.age.l <- as.data.frame(scale(bs(mydat$age, df = 5, degree = 3), center = TRUE, scale = FALSE))
  colnames(bs.age.l) <- c("A1", "A2", "A3", "A4", "A5")
  mydat <- cbind(mydat, bs.age.l)
  
  
  y <- mydat$cost[((1-p.test)*l + 1):l]
  y.ind <- mydat$zero.ind[((1-p.test)*l + 1):l]
  y.norm <- mydat.n$cost.log[(nrow(mydat.n)-p.test*l+1) : nrow(mydat.n)]
  
  mydat$zero.ind[((1-p.test)*l + 1):l] <- rep(NA, p.test*l)
  mydat.n$cost.log[(nrow(mydat.n)-p.test*l+1) : nrow(mydat.n)] <- rep(NA, length(y.norm))
  
  
  formula.logit1 <- zero.ind ~ A1 + A2 + A3 + A4 + A5 + gender + income
  mod.logit1 <- glm(formula.logit1, data = mydat, family = binomial())
  
  formula.mod1 <- cost.log ~ A1 + A2 + A3 + A4 + A5 + gender + income
  mod1 <- glm(formula.mod1, data = mydat.n)
  
  mse.glm1 <- sum(mod1$residuals^2)/(nrow(mydat.n) - p.test*l - 8)
  pred.glm1 <- predict.glm(mod.logit1, newdata = mydat[((1-p.test)*l + 1) : l,], type = "response")*exp(predict.glm(mod1, newdata = mydat.n[(nrow(mydat.n) - p.test*l + 1):nrow(mydat.n),]) + mse.glm1/2)
  
  
  formula.logit2 <- zero.ind ~ A1 + A2 + A3 + A4 + A5 + gender + income + f(zip.un, model = "iid") 
  mod.logit2 <- inla(formula.logit2, data = mydat, family = "binomial", control.predictor = list(link = 1))
  
  formula.mod2 <- cost.log ~ A1 + A2 + A3 + A4 + A5 + gender + income + f(zip, model = "iid")
  mod2 <- inla(formula.mod2, data = mydat.n, control.predictor = list(link = 1))
  
  mse.zip2 <- sum((mydat.n$cost.log[1:(nrow(mydat.n) - p.test*l)] - mod2$summary.fitted.values$mean[1:(nrow(mydat.n) - p.test*l)])^2)/(nrow(mydat.n) - p.test*l - 8)
  pred.zip2 <- mod.logit2$summary.fitted.values$mean[((1-p.test)*l + 1) : l]*exp(mod2$summary.fitted.values$mean[(nrow(mydat.n) - p.test*l + 1):nrow(mydat.n)] + mse.zip2/2)
  
  
  formula.logit3 <- zero.ind ~ A1 + A2 + A3 + A4 + A5 + gender + income + f(zip, model = "besag", graph = mat.Ohio) 
  mod.logit3 <- inla(formula.logit3, data = mydat, family = "binomial", control.predictor = list(link = 1))
  
  formula.mod3 <- cost.log ~ A1 + A2 + A3 + A4 + A5 + gender + income + f(zip, model = "besag", graph = mat.Ohio)
  mod3 <- inla(formula.mod3, data = mydat.n, control.predictor = list(link = 1))
  
  mse.zip3 <- sum((mydat.n$cost.log[1:(nrow(mydat.n) - p.test*l)] - mod3$summary.fitted.values$mean[1:(nrow(mydat.n) - p.test*l)])^2)/(nrow(mydat.n) - p.test*l - 8)
  pred.zip3 <- mod.logit3$summary.fitted.values$mean[((1-p.test)*l + 1) : l]*exp(mod3$summary.fitted.values$mean[(nrow(mydat.n) - p.test*l + 1):nrow(mydat.n)] + mse.zip3/2)
  
  
  formula.logit4 <- zero.ind ~ A1 + A2 + A3 + A4 + A5 + gender + income + f(zip, model = "besag", graph = mat.Ohio) + f(zip.un, model = "iid") 
  mod.logit4 <- inla(formula.logit4, data = mydat, family = "binomial", control.predictor = list(link = 1))
  
  formula.mod4 <- cost.log ~ A1 + A2 + A3 + A4 + A5 + gender + income + f(zip, model = "besag", graph = mat.Ohio) + f(zip.un, model = "iid") 
  mod4 <- inla(formula.mod4, data = mydat.n, control.predictor = list(link = 1))
  
  mse.zip4 <- sum((mydat.n$cost.log[1:(nrow(mydat.n) - p.test*l)] - mod4$summary.fitted.values$mean[1:(nrow(mydat.n) - p.test*l)])^2)/(nrow(mydat.n) - p.test*l - 8)
  pred.zip4 <- mod.logit4$summary.fitted.values$mean[((1-p.test)*l + 1) : l]*exp(mod4$summary.fitted.values$mean[(nrow(mydat.n) - p.test*l + 1):nrow(mydat.n)] + mse.zip4/2)
  
  
  
  formula.mod5 <- cost.log ~ A1 + A2 + A3 + A4 + A5 + gender + income +  f(county.un, model = "iid")
  mod5 <- inla(formula.mod5, data = mydat.n, control.predictor = list(link = 1))
  
  formula.logit5 <- zero.ind ~ A1 + A2 + A3 + A4 + A5 + gender + income  + f(county.un, model = "iid")
  mod.logit5 <- inla(formula.logit5, data = mydat, family = "binomial", control.predictor = list(link = 1))
  
  mse.c5 <- sum((mydat.n$cost.log[1:(nrow(mydat.n) - p.test*l)] - mod5$summary.fitted.values$mean[1:(nrow(mydat.n) - p.test*l)])^2)/(nrow(mydat.n) - p.test*l - 8)
  pred.c5 <- mod.logit5$summary.fitted.values$mean[((1-p.test)*l + 1) : l]*exp(mod5$summary.fitted.values$mean[(nrow(mydat.n) - p.test*l + 1):nrow(mydat.n)] + mse.c5/2)
  
  
  formula.mod6 <- cost.log ~ A1 + A2 + A3 + A4 + A5 + gender + income +  f(county.str, model = "besag", graph = mat.Ohio.county)
  mod6 <- inla(formula.mod6, data = mydat.n, control.predictor = list(link = 1))
  
  
  formula.logit6 <- zero.ind ~ A1 + A2 + A3 + A4 + A5 + gender + income  + f(county.str, model = "besag", graph = mat.Ohio.county)
  mod.logit6 <- inla(formula.logit6, data = mydat, family = "binomial", control.predictor = list(link = 1))
  
  mse.c6 <- sum((mydat.n$cost.log[1:(nrow(mydat.n) - p.test*l)] - mod6$summary.fitted.values$mean[1:(nrow(mydat.n) - p.test*l)])^2)/(nrow(mydat.n) - p.test*l - 8)
  pred.c6 <- mod.logit6$summary.fitted.values$mean[((1-p.test)*l + 1) : l]*exp(mod6$summary.fitted.values$mean[(nrow(mydat.n) - p.test*l + 1):nrow(mydat.n)] + mse.c6/2)
  
  formula.mod7 <- cost.log ~ A1 + A2 + A3 + A4 + A5 + gender + income + f(county.str, model = "besag", graph = mat.Ohio.county) + f(county.un, model = "iid")
  mod7 <- inla(formula.mod7, data = mydat.n, control.predictor = list(link = 1))
  
  formula.logit7 <- zero.ind ~ A1 + A2 + A3 + A4 + A5 + gender + income + f(county.str, model = "besag", graph = mat.Ohio.county) + f(county.un, model = "iid")
  mod.logit7 <- inla(formula.logit7, data = mydat, family = "binomial", control.predictor = list(link = 1))
  
  mse.c7 <- sum((mydat.n$cost.log[1:(nrow(mydat.n) - p.test*l)] - mod7$summary.fitted.values$mean[1:(nrow(mydat.n) - p.test*l)])^2)/(nrow(mydat.n) - p.test*l - 8)
  pred.c7 <- mod.logit7$summary.fitted.values$mean[((1-p.test)*l + 1) : l]*exp(mod7$summary.fitted.values$mean[(nrow(mydat.n) - p.test*l + 1):nrow(mydat.n)] + mse.c7/2)
  
  out <- c(pred.glm1, pred.zip2, pred.zip3, pred.zip4, pred.c5, pred.c6, pred.c7, y)
  out
}


## compdat should contain a set of predictions the corresponding true responses
## k is the number of replicates
comp.bracket <- function(compdat, k){
  n <- nrow(compdat)/2
  pred.dat <- compdat[1:n, ]
  y.dat <- compdat[(n+1):(2*n), ]
  out <- c()
  for (i in 1:k){
    pred.i <- pred.dat[, i]
    y.i <- y.dat[, i]
    MAE.i <- mean(abs(y.i - pred.i), na.rm = TRUE)
    ME.i <- mean((y.i - pred.i), na.rm = TRUE)
    RMSE.i <- sqrt(mean((y.i - pred.i)^2, na.rm = TRUE))
    cat1 <- which(y.i == 0)
    cat2 <- which(y.i > 0 & y.i <=200)
    cat3 <- which(y.i > 200 & y.i <= 400)
    cat4 <- which(y.i > 400 & y.i <= 600)
    cat5 <- which(y.i > 600 & y.i <= 800)
    cat6 <- which(y.i > 800 & y.i <= 1000)
    cat7 <- which(y.i > 1000 & y.i <= 2000)
    cat8 <- which(y.i > 2000)
    MAE.i1 <- mean(abs(y.i[cat1] - pred.i[cat1]), na.rm = TRUE)
    ME.i1 <- mean((y.i[cat1] - pred.i[cat1]), na.rm = TRUE)
    RMSE.i1 <- sqrt(mean((y.i[cat1] - pred.i[cat1])^2, na.rm = TRUE))
    
    MAE.i2 <- mean(abs(y.i[cat2] - pred.i[cat2]), na.rm = TRUE)
    ME.i2 <- mean((y.i[cat2] - pred.i[cat2]), na.rm = TRUE)
    RMSE.i2 <- sqrt(mean((y.i[cat2] - pred.i[cat2])^2, na.rm = TRUE))
    
    MAE.i3 <- mean(abs(y.i[cat3] - pred.i[cat3]), na.rm = TRUE)
    ME.i3 <- mean((y.i[cat3] - pred.i[cat3]), na.rm = TRUE)
    RMSE.i3 <- sqrt(mean((y.i[cat3] - pred.i[cat3])^2, na.rm = TRUE))
    
    MAE.i4 <- mean(abs(y.i[cat4] - pred.i[cat4]), na.rm = TRUE)
    ME.i4 <- mean((y.i[cat4] - pred.i[cat4]), na.rm = TRUE)
    RMSE.i4 <- sqrt(mean((y.i[cat4] - pred.i[cat4])^2, na.rm = TRUE))
    
    MAE.i5 <- mean(abs(y.i[cat5] - pred.i[cat5]), na.rm = TRUE)
    ME.i5 <- mean((y.i[cat5] - pred.i[cat5]), na.rm = TRUE)
    RMSE.i5 <- sqrt(mean((y.i[cat5] - pred.i[cat5])^2, na.rm = TRUE))
    
    MAE.i6 <- mean(abs(y.i[cat6] - pred.i[cat6]), na.rm = TRUE)
    ME.i6 <- mean((y.i[cat6] - pred.i[cat6]), na.rm = TRUE)
    RMSE.i6 <- sqrt(mean((y.i[cat6] - pred.i[cat6])^2, na.rm = TRUE))
    
    MAE.i7 <- mean(abs(y.i[cat7] - pred.i[cat7]), na.rm = TRUE)
    ME.i7 <- mean((y.i[cat7] - pred.i[cat7]), na.rm = TRUE)
    RMSE.i7 <- sqrt(mean((y.i[cat7] - pred.i[cat7])^2, na.rm = TRUE))
    
    MAE.i8 <- mean(abs(y.i[cat8] - pred.i[cat8]), na.rm = TRUE)
    ME.i8 <- mean((y.i[cat8] - pred.i[cat8]), na.rm = TRUE)
    RMSE.i8 <- sqrt(mean((y.i[cat8] - pred.i[cat8])^2, na.rm = TRUE))
    out.i <- c(MAE.i, ME.i, RMSE.i, MAE.i1, ME.i1, RMSE.i1, MAE.i2, ME.i2, RMSE.i2, 
               MAE.i3, ME.i3, RMSE.i3, MAE.i4, ME.i4, RMSE.i4, MAE.i5, ME.i5, RMSE.i5, 
               MAE.i6, ME.i6, RMSE.i6, MAE.i7, ME.i7, RMSE.i7, MAE.i8, ME.i8, RMSE.i8)
    out <- rbind(out, out.i) 
    out <- as.data.frame(out)
    colnames(out) <- c("MAE", "ME", "RMSE", "MAE.cat1", "ME.cat1", "RMSE.cat1", 
                       "MAE.cat2", "ME.cat2", "RMSE.cat2", "MAE.cat3", "ME.cat3", "RMSE.cat3", 
                       "MAE.cat4", "ME.cat4", "RMSE.cat4", "MAE.cat5", "ME.cat5", "RMSE.cat5", 
                       "MAE.cat6", "ME.cat6", "RMSE.cat6", "MAE.cat7", "ME.cat7", "RMSE.cat7", 
                       "MAE.cat8", "ME.cat8", "RMSE.cat8")
    
  }
  out
}

k = 100
p.test = 0.25
system.time(predict.comp <- replicate(k, spatial.boot(mydat.pred, p.test), simplify = "matrix"))

N = nrow(predict.comp)
n.p = N/8
glm1 <- rbind(predict.comp[1:n.p, ], predict.comp[(7*n.p + 1): N, ])
ziid <- rbind(predict.comp[(n.p + 1):(2*n.p), ], predict.comp[(7*n.p + 1): N, ])
zstr <- rbind(predict.comp[(2*n.p + 1):(3*n.p), ], predict.comp[(7*n.p + 1): N, ])
zspatial <- rbind(predict.comp[(3*n.p + 1):(4*n.p), ], predict.comp[(7*n.p + 1): N, ])
ciid <- rbind(predict.comp[(4*n.p + 1):(5*n.p), ], predict.comp[(7*n.p + 1): N, ])
cstr <- rbind(predict.comp[(5*n.p + 1):(6*n.p), ], predict.comp[(7*n.p + 1): N, ])
cspatial <- rbind(predict.comp[(6*n.p + 1):(7*n.p), ], predict.comp[(7*n.p + 1): N, ])

glm.out <- comp.bracket(glm1, k)
ziid.out <- comp.bracket(ziid, k)
zstr.out <- comp.bracket(zstr, k)
zspatial.out <- comp.bracket(zspatial, k)
ciid.out <- comp.bracket(ciid, k)
cstr.out <- comp.bracket(cstr, k)
cspatial.out <- comp.bracket(cspatial, k)


glm.mean <- apply(glm.out, 2, mean)
MAE.mean.glm <- glm.mean[c(4, 7, 10, 13, 16, 19, 22, 25)]
ME.mean.glm <- glm.mean[c(5, 8, 11, 14, 17, 20, 23, 26)]
RMSE.mean.glm <- glm.mean[c(6, 9, 12, 15, 18, 21, 24, 27)]

zip.mean <- apply(zspatial.out, 2, mean)
MAE.mean.zip <- zip.mean[c(4, 7, 10, 13, 16, 19, 22, 25)]
ME.mean.zip <- zip.mean[c(5, 8, 11, 14, 17, 20, 23, 26)]
RMSE.mean.zip <- zip.mean[c(6, 9, 12, 15, 18, 21, 24, 27)]
zip.mean.iid <- apply(ziid.out, 2, mean)
zip.mean.str <- apply(zstr.out, 2, mean)

county.mean <- apply(cspatial.out, 2, mean)
MAE.mean.county <- county.mean[c(4, 7, 10, 13, 16, 19, 22, 25)]
ME.mean.county <- county.mean[c(5, 8, 11, 14, 17, 20, 23, 26)]
RMSE.mean.county <- county.mean[c(6, 9, 12, 15, 18, 21, 24, 27)]
county.mean.iid <- apply(ciid.out, 2, mean)
county.mean.str <- apply(cstr.out, 2, mean)


glm.sd <- apply(glm.out, 2, sd)
zip.sd <- apply(zspatial.out, 2, sd)
county.sd <- apply(cspatial.out, 2, sd)




