## County Level Analysis
## Model 5: model with county level unstructured effect only
formula.mod.c.comp <- cost.log ~ A1 + A2 + A3 + A4 + A5 + gender + income +  f(county.un, model = "iid")
mod5 <- inla(formula.mod.c.comp, data = mydat.n, control.compute = list(dic = TRUE, cpo = TRUE))

formula.logit.t.c.comp <- zero.ind ~ A1 + A2 + A3 + A4 + A5 + gender + income  + f(county.un, model = "iid")
mod.logit5 <- inla(formula.logit.t.c.comp, data = mydat, family = "binomial", control.compute = list(dic = TRUE, cpo = TRUE))


## Model 6: model with county level structured effect only
formula.mod.c.comp1 <- cost.log ~ A1 + A2 + A3 + A4 + A5 + gender + income +  f(county.str, model = "besag", graph = mat.Ohio.county)
mod6 <- inla(formula.mod.c.comp1, data = mydat.n, control.compute = list(dic = TRUE, cpo = TRUE))

formula.logit.t.c.comp1 <- zero.ind ~ A1 + A2 + A3 + A4 + A5 + gender + income  + f(county.str, model = "besag", graph = mat.Ohio.county)
mod.logit6 <- inla(formula.logit.t.c.comp1, data = mydat, family = "binomial", control.compute = list(dic = TRUE, cpo = TRUE))


## Model 7: model with both county level structured and unstructured effects
formula.mod.c <- cost.log ~ A1 + A2 + A3 + A4 + A5 + gender + income + f(county.str, model = "besag", graph = mat.Ohio.county) + f(county.un, model = "iid")
mod7 <- inla(formula.mod.c, data = mydat.n, control.compute = list(dic = TRUE, cpo = TRUE))

formula.logit.t.c <- zero.ind ~ A1 + A2 + A3 + A4 + A5 + gender + income + f(county.str, model = "besag", graph = mat.Ohio.county) + f(county.un, model = "iid")
mod.logit7 <- inla(formula.logit.t.c, data = mydat, family = "binomial", control.compute = list(dic = TRUE, cpo = TRUE))


## Plot fitted structured spatial effects on map
pdf("./sp_fit_county.pdf", width = 7, height = 3.4)
#png("./sp_fit_county.png", width = 504, height = 220)
par(mfrow = c(1, 2), mgp = c(0, 0, 0), mar = c(0, 0, 0, 0), ps = 9)
options(scipen = 100)
X.county$county.l <- mod.logit7$summary.random$county.str$mean
colors <- brewer.pal(9, "Greys")
brks.s <- classIntervals(X.county$county.l, n = 9, style = "quantile")
brks.S <- brks.s$brks
colcode <- findInterval(X.county$county.l, brks.S, all.inside = TRUE)
plot(X.county, col = colors[colcode], axes = F)

for(i in 1:9){
  int.c <- paste("[", signif(brks.S[i], digits = 2), ",", signif(brks.S[i+1], digits = 2), ")", sep = "")
  if(i == 1) c <- int.c
  else c <- c(c, int.c)
}
legend("bottomright", 
       legend = rev(c), 
       title = "Effects",
       fill = names(table(colors[colcode])),
       bty = "n", cex = 0.5)

X.county$county.n <- mod7$summary.random$county.str$mean
colors <- brewer.pal(9, "Greys")
brks.s <- classIntervals(X.county$county.n, n = 9, style = "quantile")
brks.S <- brks.s$brks
colcode <- findInterval(X.county$county.n, brks.S, all.inside = TRUE)
plot(X.county, col = colors[colcode], axes = F)

for(i in 1:9){
  int.c <- paste("[", signif(brks.S[i], digits = 2), ",", signif(brks.S[i+1], digits = 2), ")", sep = "")
  if(i == 1) c <- int.c
  else c <- c(c, int.c)
}
legend("bottomright", 
       legend = rev(c), 
       title = "Effects",
       fill = names(table(colors[colcode])),
       bty = "n", cex = 0.5)


dev.off()

## Construct final ratings and plot on map

coef.l <- mod.logit7$summary.fixed[, 1]
design.X <- cbind(1, mydat$A1, mydat$A2, mydat$A3, mydat$A4, mydat$A5, mydat$gender, mydat$income)[1:20000, ]
eta <- as.data.frame(design.X %*% coef.l)
eta$county <- mydat$County[1:20000]
colnames(eta) <- c("eta", "county")
eta.mean <- ddply(eta, ~county, summarise, mean = mean(eta))
eta.mean$index <- match(eta.mean$county,  X.county$NAME10)
mean.eta <- mean(eta$eta)
index <- seq(1,length(county), 1)
eta.m <- rep(mean.eta, length(county))
eta.m <- as.data.frame(cbind(index, eta.m))
eta.m[eta.mean$index, ]$eta.m <- eta.mean$mean
eta.l <- eta.m$eta.m + mod.logit7$summary.random$county.str$mean
p.l <- binomial()$linkinv(eta.l)
final.sp <- p.l * exp(X.county$county.n)
X.county$final.sp <- final.sp

pdf("./final_county.pdf", width = 7, height = 5.833)
#png("./final_county.png", width = 300, height = 250)
par(mgp = c(0, 0, 0), mar = c(0, 0, 0, 0), ps = 9)
colors <- brewer.pal(9, "Greys")
brks.f <- classIntervals(X.county$final.sp, n = 9, style = "quantile")
brks.F <- brks.f$brks
colcode <- findInterval(X.county$final.sp, brks.F, all.inside = TRUE)
plot(X.county, col = colors[colcode], axes = F)

for(i in 1:9){
  int.c <- paste("[", signif(brks.F[i], digits = 2), ",", signif(brks.F[i+1], digits = 2), ")", sep = "")
  if(i == 1) c <- int.c
  else c <- c(c, int.c)
}
legend("bottomright", 
       legend = rev(c), 
       title = "Effects",
       fill = names(table(colors[colcode])),
       bty = "n", cex = 1)
dev.off()




