## Zipcode Level Analysis

options(scipen = 100)

## Logistic part
## Note that the prior values can be specified by users in the formula part, 
## for example: formula <- zero.ind ~ A1 + A2 + A3 + A4 + A5 + gender + f(income, model = "linear", mean.linear = 1, prec.linear = 1)
## Model 1: glm model
formula.logit1 <- zero.ind ~ A1 + A2 + A3 + A4 + A5 + gender + income
mod.logit1 <- glm(formula.logit1, data = mydat, family = binomial())

## Model 2: model with zipcode level unstructured effect only
formula.logit2 <- zero.ind ~ A1 + A2 + A3 + A4 + A5 + gender + income + f(zip.un, model = "iid") 
mod.logit2 <- inla(formula.logit2, data = mydat, family = "binomial", control.compute = list(dic = TRUE, cpo = TRUE))

## Model 3: model with zipcode level structured effect only
formula.logit3 <- zero.ind ~ A1 + A2 + A3 + A4 + A5 + gender + income + f(zip, model = "besag", graph = mat.Ohio) 
mod.logit3 <- inla(formula.logit3, data = mydat, family = "binomial", control.compute = list(dic = TRUE, cpo = TRUE))

## Model 4: model with both zipcode level structured and unstructured effects (true model)
formula.logit4 <- zero.ind ~ A1 + A2 + A3 + A4 + A5 + gender + income + f(zip, model = "besag", graph = mat.Ohio) + f(zip.un, model = "iid") 
mod.logit4 <- inla(formula.logit4, data = mydat, family = "binomial", control.compute = list(dic = TRUE, cpo = TRUE))



## Normal part
## Model 1: glm model
formula.mod1 <- cost.log ~ A1 + A2 + A3 + A4 + A5 + gender + income + factor(county.un)
mod4 <- glm(formula.mod4, data = mydat.n)

## Model 2: model with zipcode level unstructured effect only
formula.mod2 <- cost.log ~ A1 + A2 + A3 + A4 + A5 + gender + income + f(zip, model = "iid")
mod2 <- inla(formula.mod2, data = mydat.n, control.predictor = list(link = 1), control.compute = list(dic = TRUE, cpo = TRUE))

## Model 3: model with zipcode level structured effect only
formula.mod3 <- cost.log ~ A1 + A2 + A3 + A4 + A5 + gender + income + f(zip, model = "besag", graph = mat.Ohio)
mod3 <- inla(formula.mod3, data = mydat.n, control.predictor = list(link = 1), control.compute = list(dic = TRUE, cpo = TRUE))

## Model 4: model with both zipcode level structured and unstructured effects (true model)
formula.mod4 <- cost.log ~ A1 + A2 + A3 + A4 + A5 + gender + income + f(zip, model = "besag", graph = mat.Ohio) + f(zip.un, model = "iid") 
mod4 <- inla(formula.mod4, data = mydat.n, control.predictor = list(link = 1), control.compute = list(dic = TRUE, cpo = TRUE))


## Plot age effects and corresponding CIs
coef.age.n <- mod4$summary.fixed[2:6, 1]
coef.age.n.l <- mod4$summary.fixed[2:6, 3]
coef.age.n.u <- mod4$summary.fixed[2:6, 5]
pred.age.n <- as.matrix(bs.age.n) %*% coef.age.n
lower.age.n <- as.matrix(bs.age.n) %*% coef.age.n.l
upper.age.n <- as.matrix(bs.age.n) %*% coef.age.n.u

coef.age.l <- mod.logit4$summary.fixed[2:6, 1]
coef.age.l.l <- mod.logit4$summary.fixed[2:6, 3]
coef.age.l.u <- mod.logit4$summary.fixed[2:6, 5]
pred.age.l <- as.matrix(bs.age.l) %*% coef.age.l
lower.age.l <- as.matrix(bs.age.l) %*% coef.age.l.l
upper.age.l <- as.matrix(bs.age.l) %*% coef.age.l.u

plot.age.n <- cbind(mydat.n$age, pred.age.n, lower.age.n, upper.age.n)
plot.age.l <- cbind(mydat$age, pred.age.l, lower.age.l, upper.age.l)
plot.age.n <- plot.age.n[order(mydat.n$age), ]
plot.age.l <- plot.age.l[order(mydat$age), ]

age.eff.l <- cbind(mydat$age, mydat$age.eff)
age.eff.l <- age.eff.l[order(mydat$age), ]

age.eff.n <- cbind(mydat.n$age, mydat.n$age.eff)
age.eff.n[, 2] <- scale(age.eff.n[,2], center = TRUE, scale = FALSE)
age.eff.n <- age.eff.n[order(mydat.n$age), ]

pdf("./age_fit.pdf", width = 7, height = 3)
par(mfrow = c(1, 2), mgp = c(1.5, 0.5, 0), mar = c(2.5, 2.5, 0.1, 0.1), ps = 9)

plot(plot.age.l[, 1], plot.age.l[, 2], type = "n", xlab = "Age", ylab = "Age Effect (Logistic Model)")
lines(plot.age.l[, 1], plot.age.l[, 2])
lines(plot.age.l[, 1], plot.age.l[, 3], lty = 2)
lines(plot.age.l[, 1], plot.age.l[, 4], lty = 2)
lines(age.eff.l[, 1], age.eff.l[, 2], lty = 3)

plot(plot.age.n[, 1], plot.age.n[, 2], type = "n", xlab = "Age", ylab = "Age Effect (Normal Model)")
lines(plot.age.n[, 1], plot.age.n[, 2])
lines(plot.age.n[, 1], plot.age.n[, 3], lty = 2)
lines(plot.age.n[, 1], plot.age.n[, 4], lty = 2)
lines(age.eff.n[, 1], age.eff.n[, 2], lty = 3)

dev.off()


## Plot fitted spatial effects
pdf("./sp_fit.pdf", width = 7, height = 3.1)
#png("./sp.fit.png", width = 504, height = 220)
par(mfrow = c(1, 2), mgp = c(0, 0, 0), mar = c(0, 0, 0, 0), ps = 9)
options(scipen = 100)
## Plot fitted logistic structured spatial effects on map
mod.logit4$summary.random$zip$mean[position.0] <- 0
X$zip.logit <- mod.logit4$summary.random$zip$mean
colors <- brewer.pal(9, "Greys")
brks.l <- classIntervals(X$zip.logit, n = 9, style = "quantile")
brks.L <- brks.l$brks
colcode <- findInterval(X$zip.logit, brks.L, all.inside = TRUE)
plot(X, col = colors[colcode], axes = F)

for(i in 1:9){
  int.c <- paste("[", signif(brks.L[i], digits = 2), ",", signif(brks.L[i+1], digits = 2), ")", sep = "")
  if(i == 1) c <- int.c
  else c <- c(c, int.c)
}
legend("bottomright", 
       legend = rev(c), 
       title = "Effects",
       fill = names(table(colors[colcode])),
       bty = "n", cex = 0.5)


## Plot fitted normal structured spatial effects on map
mod4$summary.random$zip$mean[position.0] <- 0
X$zip.s <- mod4$summary.random$zip$mean
colors <- brewer.pal(9, "Greys")
brks.s <- classIntervals(X$zip.s, n = 9, style = "quantile")
brks.S <- brks.s$brks
colcode <- findInterval(X$zip.s, brks.S, all.inside = TRUE)
plot(X, col = colors[colcode], axes = F)

for(i in 1:9){
  int.c <- paste("[", signif(brks.S[i], digits = 2), ",", signif(brks.S[i+1], digits = 2), ")", sep = "")
  if(i == 1) c <- int.c
  else c <- c(c, int.c)
}
legend("bottomright", 
       legend = rev(c), 
       title = "Effects",
       fill = names(table(colors[colcode])),
       bty = "n", cex = 0.5)
dev.off()


## Plot fitted normal unstructured spatial effects on map
#X$zip.u <- rep(0, 1197)
#X$zip.u[mod4$summary.random$zip.un$ID] <- mod4$summary.random$zip.un$mean
#colors <- brewer.pal(9, "Greys")
#brks.u <- classIntervals(X$zip.u, n = 9, style = "quantile")
#brks.U <- brks.u$brks
#colcode <- findInterval(X$zip.u, brks.U, all.inside = TRUE)
#plot(X, col = colors[colcode], axes = F)

#for(i in 1:9){
#  int.c <- paste("[", signif(brks.U[i], digits = 2), ",", signif(brks.U[i+1], digits = 2), ")", sep = "")
#  if(i == 1) c <- int.c
#  else c <- c(c, int.c)
#}
#legend("bottomright", 
#       legend = rev(c), 
#       title = "Effects",
#       fill = names(table(colors[colcode])),
#       bty = "n", cex = 0.5)


## Construct maps for confidence intervals of normal spatial effects
#m <- mod4$marginals.random$zip
#zip.hpd9 <- zip.hpd8 <- matrix(1:2, 1, 2)
#for (i in 1:1197){
#  zip.hpd9 <- rbind(zip.hpd9, inla.hpdmarginal(0.9, m[[i]]))
#  zip.hpd8 <- rbind(zip.hpd8, inla.hpdmarginal(0.8, m[[i]]))
#}
#zip.hpd9 <- as.data.frame(zip.hpd9[-1, ])
#zip.hpd8 <- as.data.frame(zip.hpd8[-1, ])

#zip.hpd8$sig <- 0
#zip.hpd8[zip.hpd8$low > 0,]$sig <- 1
#zip.hpd8[zip.hpd8$high < 0,]$sig <- -1

#zip.hpd9$sig <- 0
#zip.hpd9[zip.hpd9$low > 0,]$sig <- 1
#zip.hpd9[zip.hpd9$high < 0,]$sig <- -1

#pdf("./HPD.pdf", width = 7, height = 3.1)
#png("./HPD.png", width = 504, height = 220)
#par(mfrow = c(1, 2), mgp = c(0, 0, 0), mar = c(0, 0, 0, 0), ps = 9)
#X$hpd8 <- zip.hpd8$sig
#colors <- brewer.pal(3, "Greys")
#brks.s <- classIntervals(X$hpd8, n = 3, style = "quantile")
#brks.s <- brks.s$brks
#plot(X, col = colors[findInterval(X$hpd8, brks.s, all.inside = TRUE)], axes = F)

#X$hpd9 <- zip.hpd9$sig
#colors <- brewer.pal(3, "Greys")
#brks.s <- classIntervals(X$hpd9, n = 3, style = "quantile")
#brks.s <- brks.s$brks
#plot(X, col = colors[findInterval(X$hpd9, brks.s, all.inside = TRUE)], axes = F)
#dev.off()



## Construct final ratings and plot on map

coef.l <- mod.logit4$summary.fixed[, 1]
design.X <- cbind(1, mydat$A1, mydat$A2, mydat$A3, mydat$A4, mydat$A5, mydat$gender, mydat$income)
eta <- as.data.frame(design.X %*% coef.l)
eta$zip <- mydat$zipcode
colnames(eta) <- c("eta", "zip")
eta.mean <- ddply(eta, ~zip, summarise, mean = mean(eta))
eta.mean$index <- match(eta.mean$zip,  X$ZCTA5CE10)
mean.eta <- mean(eta$eta)
index <- seq(1,length(zip), 1)
eta.m <- rep(mean.eta, length(zip))
eta.m <- as.data.frame(cbind(index, eta.m))
eta.m[eta.mean$index, ]$eta.m <- eta.mean$mean
eta.l <- eta.m$eta.m + mod.logit4$summary.random$zip$mean
p.l <- binomial()$linkinv(eta.l)
final.sp <- p.l * exp(X$zip.s)
X$final.sp <- final.sp

pdf("./final.pdf", width = 7, height = 5.833)
#png("./final.png", width = 300, height = 250)
par(mgp = c(0, 0, 0), mar = c(0, 0, 0, 0), ps = 9)
colors <- brewer.pal(9, "Greys")
brks.f <- classIntervals(X$final.sp, n = 9, style = "quantile")
brks.F <- brks.f$brks
colcode <- findInterval(X$final.sp, brks.F, all.inside = TRUE)
plot(X, col = colors[colcode], axes = F)

for(i in 1:9){
  int.c <- paste("[", signif(brks.F[i], digits = 2), ",", signif(brks.F[i+1], digits = 2), ")", sep = "")
  if(i == 1) c <- int.c
  else c <- c(c, int.c)
}
legend("bottomright", 
       legend = rev(c), 
       title = "Effects",
       fill = names(table(colors[colcode])),
       bty = "n", cex = 1)
dev.off()







